@@ -44,6 +44,11 @@ FieldMeta *ObjectPad_mop_create_field(pTHX_ SV *fieldname, FIELDOFFSET fieldix,
   return fieldmeta;
 }
 
+ClassMeta *ObjectPad_mop_field_get_class(pTHX_ FieldMeta *fieldmeta)
+{
+  return fieldmeta->class;
+}
+
 SV *ObjectPad_mop_field_get_name(pTHX_ FieldMeta *fieldmeta)
 {
   return fieldmeta->name;
@@ -172,9 +177,16 @@ static void register_field_attribute(const char *name, const struct FieldHookFun
   fieldattrs = reg;
 }
 
-void ObjectPad_mop_field_apply_attribute(pTHX_ FieldMeta *fieldmeta, const char *name, SV *value)
+enum {
+  APPLY_ATTRIBUTE_PARSE             = (1<<0),
+  APPLY_ATTRIBUTE_USE_RUNTIME_HINTS = (1<<1),
+};
+
+static void apply_attribute(pTHX_ FieldMeta *fieldmeta, const char *name, SV *value, U8 flags)
 {
+  bool use_runtime_hints = flags & APPLY_ATTRIBUTE_USE_RUNTIME_HINTS;
   HV *hints = GvHV(PL_hintgv);
+  COPHH *cophh = CopHINTHASH_get(PL_curcop);
 
   if(value && (!SvPOK(value) || !SvCUR(value)))
     value = NULL;
@@ -184,9 +196,16 @@ void ObjectPad_mop_field_apply_attribute(pTHX_ FieldMeta *fieldmeta, const char
     if(!strEQ(name, reg->name))
       continue;
 
-    if(reg->funcs->permit_hintkey &&
-       (!hints || !hv_fetch(hints, reg->funcs->permit_hintkey, reg->permit_hintkeylen, 0)))
-      continue;
+    if(reg->funcs->permit_hintkey) {
+      if(use_runtime_hints) {
+        if(!cophh_fetch_pvn(cophh, reg->funcs->permit_hintkey, reg->permit_hintkeylen, 0, 0))
+          continue;
+      }
+      else {
+        if(!hints || !hv_fetch(hints, reg->funcs->permit_hintkey, reg->permit_hintkeylen, 0))
+          continue;
+      }
+    }
 
     break;
   }
@@ -199,6 +218,9 @@ void ObjectPad_mop_field_apply_attribute(pTHX_ FieldMeta *fieldmeta, const char
   if((reg->funcs->flags & OBJECTPAD_FLAG_ATTR_MUST_VALUE) && !value)
     croak("Attribute :%s requires a value", name);
 
+  if((flags & APPLY_ATTRIBUTE_PARSE) && reg->funcs->parse)
+    value = (*reg->funcs->parse)(aTHX_ fieldmeta, value, reg->funcdata);
+
   SV *attrdata = value;
 
   if(reg->funcs->apply) {
@@ -224,6 +246,17 @@ void ObjectPad_mop_field_apply_attribute(pTHX_ FieldMeta *fieldmeta, const char
   av_push(fieldmeta->hooks, (SV *)hook);
 }
 
+void ObjectPad_mop_field_apply_attribute(pTHX_ FieldMeta *fieldmeta, const char *name, SV *value)
+{
+  bool runtime = !IN_PERL_COMPILETIME;
+  apply_attribute(aTHX_ fieldmeta, name, value, runtime ? APPLY_ATTRIBUTE_USE_RUNTIME_HINTS : 0);
+}
+
+void ObjectPad_mop_field_parse_and_apply_attribute(pTHX_ FieldMeta *fieldmeta, const char *name, SV *value)
+{
+  apply_attribute(aTHX_ fieldmeta, name, value, APPLY_ATTRIBUTE_PARSE);
+}
+
 static FieldAttributeRegistration *get_active_registration(pTHX_ const char *name)
 {
   COPHH *cophh = CopHINTHASH_get(PL_curcop);
@@ -297,11 +330,13 @@ AV *ObjectPad_mop_field_get_attribute_values(pTHX_ FieldMeta *fieldmeta, const c
   return ret;
 }
 
-SV *ObjectPad_get_obj_fieldsv(pTHX_ SV *self, ClassMeta *classmeta, FieldMeta *fieldmeta)
+SV *ObjectPad_get_obj_fieldsv(pTHX_ SV *self, FieldMeta *fieldmeta)
 {
   SV *fieldstore;
   FIELDOFFSET fieldix;
 
+  ClassMeta *classmeta = fieldmeta->class;
+
   assert(SvROK(self));
   assert(SvOBJECT(SvRV(self)));
 
@@ -367,6 +402,8 @@ static OP *S_newFIELDSVOP(pTHX_ U32 flags, FIELDOFFSET fieldix)
 {
   OP *o = newOP_CUSTOM(&pp_fieldsv, flags);
   o->op_targ = fieldix;
+  if(flags & OPfMETHSTART_ROLE)
+    o->op_flags |= OPf_SPECIAL;
   return o;
 }
 
@@ -374,7 +411,7 @@ static OP *S_newFIELDSVOP(pTHX_ U32 flags, FIELDOFFSET fieldix)
 static OP *S_gen_field_init_op(pTHX_ FieldMeta *fieldmeta)
 {
   ClassMeta *classmeta = fieldmeta->class;
-  U8 opf_special_if_role = (classmeta->type == METATYPE_ROLE) ? OPf_SPECIAL : 0;
+  U32 opflags_if_role = (classmeta->type == METATYPE_ROLE) ? OPfMETHSTART_ROLE : 0;
 
   char sigil = SvPV_nolen(fieldmeta->name)[0];
   OP *op = NULL;
@@ -413,11 +450,33 @@ static OP *S_gen_field_init_op(pTHX_ FieldMeta *fieldmeta)
           valueop = newHELEMEXISTSOROP(OPpHELEMEXISTSOR_DELETE << 8, helemop, valueop);
       }
 
-      if(valueop)
+      if(valueop) {
         op = newBINOP(OP_SASSIGN, 0,
           valueop,
           /* $fields[$idx] */
-          newFIELDSVOP(OPf_MOD | opf_special_if_role, fieldmeta->fieldix));
+          newFIELDSVOP(OPf_MOD | opflags_if_role, fieldmeta->fieldix));
+
+        /* Can't just
+         *   MOP_FIELD_RUN_HOOKS(fieldmeta, gen_valueassert_op, ...)
+         * because of collecting up the return values
+         */
+        U32 hooki;
+        for(hooki = 0; fieldmeta->hooks && hooki < av_count(fieldmeta->hooks); hooki++) {
+          struct FieldHook *h = (struct FieldHook *)AvARRAY(fieldmeta->hooks)[hooki];         \
+          if(!h->funcs->gen_valueassert_op)
+            continue;
+
+          OP *assertop = (*h->funcs->gen_valueassert_op)(aTHX_ fieldmeta, h->attrdata, h->funcdata,
+            newFIELDSVOP(opflags_if_role, fieldmeta->fieldix));
+
+          if(!assertop)
+            continue;
+
+          op = op_append_elem(OP_LINESEQ, op,
+            assertop);
+        }
+      }
+
       break;
     }
     case '@':
@@ -433,7 +492,7 @@ static OP *S_gen_field_init_op(pTHX_ FieldMeta *fieldmeta)
       if(valueop) {
         /* $fields[$idx]->@* or ->%* */
         OP *lhs = force_list_keeping_pushmark(newUNOP(coerceop, OPf_MOD|OPf_REF,
-                    newFIELDSVOP(opf_special_if_role, fieldmeta->fieldix)));
+                    newFIELDSVOP(opflags_if_role, fieldmeta->fieldix)));
 
         op = newBINOP(OP_AASSIGN, 0,
             force_list_keeping_pushmark(valueop),
@@ -569,7 +628,7 @@ static void S_generate_field_accessor_method(pTHX_ FieldMeta *fieldmeta, SV *mna
   ENTER;
 
   ClassMeta *classmeta = fieldmeta->class;
-  U8 opf_special_if_role = (classmeta->type == METATYPE_ROLE ? OPf_SPECIAL : 0);
+  U32 opflags_if_role = (classmeta->type == METATYPE_ROLE) ? OPfMETHSTART_ROLE : 0;
   char sigil = SvPVX(fieldmeta->name)[0];
 
   SV *mname_fq = newSVpvf("%" SVf "::%" SVf, classmeta->name, mname);
@@ -596,8 +655,8 @@ static void S_generate_field_accessor_method(pTHX_ FieldMeta *fieldmeta, SV *mna
     newSTATEOP(0, NULL, NULL));
   OP *methstartop;
   ops = op_append_list(OP_LINESEQ, ops,
-    methstartop = newMETHSTARTOP(0 |
-      opf_special_if_role |
+    methstartop = newMETHSTARTOP(OPf_STACKED |
+      opflags_if_role |
       (classmeta->repr << 8)));
 
   int req_args = 0;
@@ -644,7 +703,7 @@ static void S_generate_field_accessor_method(pTHX_ FieldMeta *fieldmeta, SV *mna
 #else
   {
     ops = op_append_list(OP_LINESEQ, ops,
-      newFIELDPADOP(private << 8 | opf_special_if_role, padix, fieldix));
+      newFIELDPADOP(private << 8 | opflags_if_role, padix, fieldix));
   }
 #endif
 
@@ -845,9 +904,22 @@ static bool fieldhook_inheritble_apply(pTHX_ FieldMeta *fieldmeta, SV *value, SV
 
 static struct FieldHookFuncs fieldhooks_inheritable = {
   .ver   = OBJECTPAD_ABIVERSION,
+  .flags = OBJECTPAD_FLAG_ATTR_NO_VALUE,
   .apply = &fieldhook_inheritble_apply,
 };
 
+struct FieldHookFuncs_v76 {
+  U32 ver;
+  U32 flags;
+  const char *permit_hintkey;
+  bool (*apply)(pTHX_ FieldMeta *fieldmeta, SV *value, SV **attrdata_ptr, void *funcdata);
+  void (*seal)(pTHX_ FieldMeta *fieldmeta, SV *attrdata, void *funcdata);
+  void (*gen_accessor_ops)(pTHX_ FieldMeta *fieldmeta, SV *attrdata, void *funcdata,
+          enum AccessorType type, struct AccessorGenerationCtx *ctx);
+  void (*post_makefield)(pTHX_ FieldMeta *fieldmeta, SV *attrdata, void *funcdata, SV *field);
+  void (*post_construct)(pTHX_ FieldMeta *fieldmeta, SV *attrdata, void *funcdata, SV *field);
+};
+
 void ObjectPad_register_field_attribute(pTHX_ const char *name, const struct FieldHookFuncs *funcs, void *funcdata)
 {
   if(funcs->ver < 57)
@@ -863,6 +935,26 @@ void ObjectPad_register_field_attribute(pTHX_ const char *name, const struct Fie
   if(!funcs->permit_hintkey)
     croak("Third-party field attributes require a permit hinthash key");
 
+  if(funcs->ver < OBJECTPAD_ABIVERSION) {
+    const struct FieldHookFuncs_v76 *funcs_v76 = (const struct FieldHookFuncs_v76 *)funcs;
+
+    struct FieldHookFuncs *funcs_v810;
+    Newx(funcs_v810, 1, struct FieldHookFuncs);
+
+    *funcs_v810 = (struct FieldHookFuncs){
+      .ver              = OBJECTPAD_ABIVERSION,
+      .flags            = funcs_v76->flags,
+      .permit_hintkey   = funcs_v76->permit_hintkey,
+      .apply            = funcs_v76->apply,
+      .seal             = funcs_v76->seal,
+      .gen_accessor_ops = funcs_v76->gen_accessor_ops,
+      .post_makefield   = funcs_v76->post_makefield,
+      .post_construct   = funcs_v76->post_construct,
+    };
+
+    funcs = funcs_v810;
+  }
+
   register_field_attribute(name, funcs, funcdata);
 }
 
